@@ -818,14 +818,6 @@ where
         }
     }
 
-    #[emitter]
-    fn emit_declaration_name(&mut self, n: &DeclarationName) -> Result {
-        match n {
-            DeclarationName::Ident(n) => emit!(self, n),
-            DeclarationName::DashedIdent(n) => emit!(self, n),
-        }
-    }
-
     #[emitter]
     fn emit_declaration(&mut self, n: &Declaration) -> Result {
         emit!(self, n.name);
@@ -861,14 +853,12 @@ where
             )?;
         }
 
-        if let Some(tok) = n.important {
+        if n.important.is_some() {
             if !is_custom_property {
                 formatting_space!(self);
             }
 
-            punct!(self, tok, "!");
-
-            self.wr.write_raw(Some(tok), "important")?;
+            emit!(self, n.important);
         }
 
         if self.ctx.semi_after_property {
@@ -876,6 +866,25 @@ where
         }
     }
 
+    #[emitter]
+    fn emit_declaration_name(&mut self, n: &DeclarationName) -> Result {
+        match n {
+            DeclarationName::Ident(n) => emit!(self, n),
+            DeclarationName::DashedIdent(n) => emit!(self, n),
+        }
+    }
+
+    #[emitter]
+    fn emit_important_flag(&mut self, n: &ImportantFlag) -> Result {
+        punct!(self, "!");
+
+        if self.config.minify {
+            self.wr.write_raw(None, "important")?;
+        } else {
+            emit!(self, n.value);
+        }
+    }
+
     #[emitter]
     fn emit_ident(&mut self, n: &Ident) -> Result {
         self.wr.write_raw(Some(n.span), &n.raw)?;
@@ -1006,7 +1015,13 @@ where
     fn emit_hex_color(&mut self, n: &HexColor) -> Result {
         punct!(self, "#");
 
-        self.wr.write_raw(Some(n.span), &n.raw)?;
+        if self.config.minify {
+            let minified = minify_hex_color(&n.value);
+
+            self.wr.write_raw(Some(n.span), &minified)?;
+        } else {
+            self.wr.write_raw(Some(n.span), &n.raw)?;
+        }
     }
 
     #[emitter]
@@ -1261,7 +1276,7 @@ where
         match n {
             SubclassSelector::Id(n) => emit!(self, n),
             SubclassSelector::Class(n) => emit!(self, n),
-            SubclassSelector::Attr(n) => emit!(self, n),
+            SubclassSelector::Attribute(n) => emit!(self, n),
             SubclassSelector::PseudoClass(n) => emit!(self, n),
             SubclassSelector::PseudoElement(n) => emit!(self, n),
             SubclassSelector::At(n) => emit!(self, n),
@@ -1270,70 +1285,98 @@ where
 
     #[emitter]
     fn emit_type_selector(&mut self, n: &TypeSelector) -> Result {
+        match n {
+            TypeSelector::TagName(n) => emit!(self, n),
+            TypeSelector::Universal(n) => emit!(self, n),
+        }
+    }
+
+    #[emitter]
+    fn emit_tag_name_selector(&mut self, n: &TagNameSelector) -> Result {
+        emit!(self, n.name);
+    }
+
+    #[emitter]
+    fn emit_universal_selector(&mut self, n: &UniversalSelector) -> Result {
         if let Some(prefix) = &n.prefix {
             emit!(self, prefix);
-            punct!(self, "|");
         }
 
-        emit!(self, n.name);
+        punct!(self, "*");
+    }
+
+    #[emitter]
+    fn emit_ns_prefix(&mut self, n: &NsPrefix) -> Result {
+        emit!(self, n.prefix);
+        punct!(self, "|");
+    }
+
+    #[emitter]
+    fn emit_wq_name(&mut self, n: &WqName) -> Result {
+        if n.prefix.is_some() {
+            emit!(self, n.prefix);
+        }
+
+        emit!(self, n.value);
     }
 
     #[emitter]
     fn emit_id_selector(&mut self, n: &IdSelector) -> Result {
         punct!(self, "#");
-        let ctx = Ctx { ..self.ctx };
-        emit!(&mut *self.with_ctx(ctx), n.text);
+        emit!(self, n.text);
     }
 
     #[emitter]
     fn emit_class_selector(&mut self, n: &ClassSelector) -> Result {
         punct!(self, ".");
-        let ctx = Ctx { ..self.ctx };
-        emit!(&mut *self.with_ctx(ctx), n.text);
+        emit!(self, n.text);
     }
 
     #[emitter]
-    fn emit_attr_selector_value(&mut self, n: &AttrSelectorValue) -> Result {
-        match n {
-            AttrSelectorValue::Str(n) => emit!(self, n),
-            AttrSelectorValue::Ident(n) => emit!(self, n),
-        }
-    }
-
-    #[emitter]
-    fn emit_attr_selector(&mut self, n: &AttrSelector) -> Result {
+    fn emit_attribute_selector(&mut self, n: &AttributeSelector) -> Result {
         punct!(self, "[");
-
-        if let Some(prefix) = &n.prefix {
-            emit!(self, prefix);
-            punct!(self, "|");
-        }
-
         emit!(self, n.name);
 
-        if let Some(matcher) = n.matcher {
-            self.wr.write_punct(None, matcher.as_str())?;
-        }
+        if n.matcher.is_some() {
+            emit!(self, n.matcher);
+            emit!(self, n.value);
 
-        emit!(self, n.value);
+            if n.modifier.is_some() {
+                match n.value {
+                    Some(AttributeSelectorValue::Str(_)) => {
+                        formatting_space!(self);
+                    }
+                    Some(AttributeSelectorValue::Ident(_)) => {
+                        space!(self);
+                    }
+                    _ => {}
+                }
 
-        if let Some(m) = &n.modifier {
-            match n.value {
-                Some(AttrSelectorValue::Str(_)) => {
-                    formatting_space!(self);
-                }
-                Some(AttrSelectorValue::Ident(_)) => {
-                    space!(self);
-                }
-                _ => {}
+                emit!(self, n.modifier);
             }
-
-            self.wr.write_raw_char(None, *m)?;
         }
 
         punct!(self, "]");
     }
 
+    #[emitter]
+    fn emit_attribute_selector_matcher(&mut self, n: &AttributeSelectorMatcher) -> Result {
+        self.wr.write_punct(None, n.value.as_str())?;
+    }
+
+    #[emitter]
+    fn emit_attribute_selector_value(&mut self, n: &AttributeSelectorValue) -> Result {
+        match n {
+            AttributeSelectorValue::Str(n) => emit!(self, n),
+            AttributeSelectorValue::Ident(n) => emit!(self, n),
+        }
+    }
+
+    #[emitter]
+    fn emit_attribute_selector_modifier(&mut self, n: &AttributeSelectorModifier) -> Result {
+        emit!(self, n.value);
+    }
+
     #[emitter]
     fn emit_nth(&mut self, n: &Nth) -> Result {
         emit!(self, n.nth);
@@ -1446,3 +1489,37 @@ where
         Ok(())
     }
 }
+
+fn minify_hex_color(value: &str) -> String {
+    let length = value.len();
+
+    if length == 6 || length == 8 {
+        let chars = value.as_bytes();
+
+        if chars[0] == chars[1] && chars[2] == chars[3] && chars[4] == chars[5] {
+            // 6 -> 3 or 8 -> 3
+            if length == 6 || chars[6] == b'f' && chars[7] == b'f' {
+                let mut minified = String::new();
+
+                minified.push((chars[0] as char).to_ascii_lowercase());
+                minified.push((chars[2] as char).to_ascii_lowercase());
+                minified.push((chars[4] as char).to_ascii_lowercase());
+
+                return minified;
+            }
+            // 8 -> 4
+            else if length == 8 && chars[6] == chars[7] {
+                let mut minified = String::new();
+
+                minified.push((chars[0] as char).to_ascii_lowercase());
+                minified.push((chars[2] as char).to_ascii_lowercase());
+                minified.push((chars[4] as char).to_ascii_lowercase());
+                minified.push((chars[6] as char).to_ascii_lowercase());
+
+                return minified;
+            }
+        }
+    }
+
+    value.to_ascii_lowercase()
+}
